"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateGetters = exports.getter = exports.camelCase = exports.inject = exports.generateCSS = exports.preprocessRules = exports.objToTuples = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Transform {key: value} object pairs into [key, value] tuples
 * @param {object} object - The object to process
 * @returns {array}
 */

var objToTuples = exports.objToTuples = function objToTuples(obj) {
  return _ramda2.default.map(function (key) {
    return [key, obj[key]];
  }, _ramda2.default.keys(obj));
};

/**
 * Generate a flat list of [selector, declaration, value] tuples
 * @param {object} rules - The rules object to process
 * @returns {array}
 */

var preprocessRules = exports.preprocessRules = _ramda2.default.compose(_ramda2.default.unnest, _ramda2.default.map(function (_ref) {
  var _ref2 = _slicedToArray(_ref, 2);

  var declaration = _ref2[0];
  var rule = _ref2[1];
  return _ramda2.default.map(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2);

    var key = _ref4[0];
    var value = _ref4[1];
    return ["." + declaration + "--" + key, declaration, value];
  }, objToTuples(rule));
}), objToTuples);

/**
 * Generate CSS string
 * @param {object} rules - The rules object to process
 * @returns {string}
 */

var generateCSS = exports.generateCSS = function generateCSS(config) {
  var rules = preprocessRules(config.rules);
  return _ramda2.default.reduce(function (acc, _ref5) {
    var _ref6 = _slicedToArray(_ref5, 3);

    var selector = _ref6[0];
    var declaration = _ref6[1];
    var value = _ref6[2];
    return "" + acc + selector + " {" + declaration + ": " + value + ";}";
  }, "", rules);
};

/**
 * Inject CSS into <head>
 * @param {string}
 * @returns {void}
 */

var inject = exports.inject = function inject(css) {
  var style = document.createElement("style");
  style.type = "text/css";
  style.appendChild(document.createTextNode(css));

  document.head.appendChild(style);
};

/**
 * Convert kebab-case to camelCase
 * @param {string}
 * @returns {string}
 */

var camelCase = exports.camelCase = function camelCase(string) {
  return _ramda2.default.replace(/-([a-z])/g, function (_, match) {
    return _ramda2.default.toUpper(match);
  }, string);
};

/**
 * Retrieve classname from config
 * @param {object} config
 * @param {string} rule
 * @param {string} key
 * @returns {string} classname
 */

var getter = exports.getter = _ramda2.default.curry(function (config, rule, key) {
  if (config.rules[rule][key]) {
    return rule + "--" + key;
  } else {
    console.error("Couldn't find '" + key + "' for '" + rule + "' rule.");
    return null;
  }
});

/**
 * Retrieve getters
 * @param {object} config
 * @returns {object} getters
 */

var generateGetters = exports.generateGetters = function generateGetters(config) {
  return _ramda2.default.reduce(function (acc, rule) {
    return _ramda2.default.assoc(camelCase(rule), getter(config, rule), acc);
  }, {}, _ramda2.default.keys(config.rules));
};

/**
 * Main
 * @param {object} config
 * @returns {object} getters
 */

exports.default = function (config) {
  inject(generateCSS(config));
  return generateGetters(config);
};